VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CSystem"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'********************************************************************************
'* Description:
'*   Exposes system-level information to be used in applications.
'*
'* Methods (Private):
'*   GetVersion - Fills module-level structure with version information
'*
'*
'* Methods (Public):
'*
'*
'* Properties:
'*   Platform (read-only) - Returns integer constant indicating Operating System Platform
'*   MajorVersion (read-only) - Returns Long containing Operating System Major Version number
'*   MinorVersion (read-only) - Returns Long containing Operating System Minor Version number
'*   BuildNumber (read-only) - Returns Long containing Operating System Build Number
'*
'*
'* Associations:
'*
'*
'*
'*
'* Events:
'*
'*
'*
'*
'* Revisions:
'*  07/28/1998 RMT Added comments.
'*
'*
'********************************************************************************

Option Explicit

' Mod INTERFACES


' Mod EVENTS


' Mod CONSTANTS
Private Const VER_PLATFORM_WIN32_NT = 2
Private Const VER_PLATFORM_WIN32_WINDOWS = 1
Private Const VER_PLATFORM_WIN32s = 0


' Mod ENUMS
Public Enum emsOSPlatform
    emsOSPlatformUnknown = 0
    emsOSPlatformWindowsNT = 1
    emsOSPlatformWindows95 = 2
    emsOSPlatformWindows98 = 3
    emsOSPlatformWindows32s = 4
End Enum


' Mod TYPES
Private Type OSVERSIONINFO
        dwOSVersionInfoSize As Long
        dwMajorVersion As Long
        dwMinorVersion As Long
        dwBuildNumber As Long
        dwPlatformId As Long
        szCSDVersion As String * 128      '  Maintenance string for PSS usage
End Type


' Mod DECLARES
Private Declare Function GetVersionEx Lib "kernel32" Alias "GetVersionExA" (lpVersionInformation As OSVERSIONINFO) As Long


' Mod VARIABLES
Private bVersionInit As Boolean
Private mosVersionInfoStruct As OSVERSIONINFO

Private Sub GetVersion()
'Fills module-level structure with version information
Dim lRet As Long
   
    mosVersionInfoStruct.dwOSVersionInfoSize = Len(mosVersionInfoStruct)
    lRet = GetVersionEx(mosVersionInfoStruct)
    
    '0 means fail
    bVersionInit = (lRet <> 0)
End Sub

Public Property Get Platform() As emsOSPlatform
'Returns constant indicating Operating System Platform

    Platform = emsOSPlatformUnknown
    
    If bVersionInit = False Then
        GetVersion
    End If
    
    If bVersionInit = True Then
        With mosVersionInfoStruct
            
            Select Case .dwPlatformId
            Case VER_PLATFORM_WIN32_NT
                Platform = emsOSPlatformWindowsNT
                
            Case VER_PLATFORM_WIN32_WINDOWS
                If .dwMinorVersion = 0 Then
                    Platform = emsOSPlatformWindows95
                ElseIf .dwMinorVersion = 10 Then
                    Platform = emsOSPlatformWindows98
                End If
                
            Case VER_PLATFORM_WIN32s
                Platform = emsOSPlatformWindows32s
                
            End Select
            
        End With
        
    End If
    
End Property

Public Property Get MajorVersion() As Long
'Returns Long containing Operating System Major Version number
    If bVersionInit = False Then
        GetVersion
    End If
    
    If bVersionInit = True Then
        MajorVersion = mosVersionInfoStruct.dwMajorVersion
    End If
    
End Property

Public Property Get MinorVersion() As Long
'Returns Long containing Operating System Minor Version number

    If bVersionInit = False Then
        GetVersion
    End If
    
    If bVersionInit = True Then
        MinorVersion = mosVersionInfoStruct.dwMinorVersion
    End If
    
End Property


Public Property Get BuildNumber() As Long
'Returns Long containing Operating System Build Number

    If bVersionInit = False Then
        GetVersion
    End If
    
    If bVersionInit = True Then
        BuildNumber = mosVersionInfoStruct.dwBuildNumber
    End If
    
End Property
