Attribute VB_Name = "modDateFunctions"
Option Explicit

Public Const TZ_REG_KEY_NT As String = "SOFTWARE\Microsoft\Windows NT\CurrentVersion\Time Zones\"
Public Const TZ_REG_KEY_95 As String = "SOFTWARE\Microsoft\Windows\CurrentVersion\Time Zones\"

Private Const MODULE As String = "modDateFunctions"

Private Const TZ_REG_ATLANTIC As String = "Atlantic Standard Time"
Private Const TZ_REG_PACIFIC As String = "Pacific Standard Time"
Private Const TZ_REG_MOUNTAIN As String = "Mountain Standard Time"
Private Const TZ_REG_USMOUNTAIN As String = "US Mountain Standard Time"
Private Const TZ_REG_CENTRAL As String = "Central Standard Time"
Private Const TZ_REG_EASTERN As String = "Eastern Standard Time"

Private Const TZ_REG_GMT As String = "GMT"

Private Const TZ_REG_ATLANTIC_95 As String = "Atlantic"
Private Const TZ_REG_PACIFIC_95 As String = "Pacific"
Private Const TZ_REG_MOUNTAIN_95 As String = "Mountain"
Private Const TZ_REG_USMOUNTAIN_95 As String = "US Mountain"
Private Const TZ_REG_CENTRAL_95 As String = "Central"
Private Const TZ_REG_EASTERN_95 As String = "Eastern"


Private Const TZ_ATLANTIC_STD_NAME As String = "Atlantic Standard Time"
Private Const TZ_ATLANTIC_DLT_NAME As String = "Atlantic Daylight Time"
Private Const TZ_PACIFIC_STD_NAME As String = "Pacific Standard Time"
Private Const TZ_PACIFIC_DLT_NAME As String = "Pacific Daylight Time"
Private Const TZ_MOUNTAIN_STD_NAME As String = "Mountain Standard Time"
Private Const TZ_MOUNTAIN_DLT_NAME As String = "Mountain Daylight Time"
Private Const TZ_USMOUNTAIN_STD_NAME As String = "US Mountain Standard Time"
Private Const TZ_USMOUNTAIN_DLT_NAME As String = "US Mountain Daylight Time"
Private Const TZ_CENTRAL_STD_NAME As String = "Central Standard Time"
Private Const TZ_CENTRAL_DLT_NAME As String = "Central Daylight Time"
Private Const TZ_EASTERN_STD_NAME As String = "Eastern Standard Time"
Private Const TZ_EASTERN_DLT_NAME As String = "Eastern Daylight Time"
Private Const TZ_GMT_STD_NAME As String = "GMT"
Private Const TZ_GMT_DLT_NAME As String = "British Summer Time"

Private Const TZ_ATLANTIC_STD_NAME_95 As String = TZ_ATLANTIC_STD_NAME
Private Const TZ_ATLANTIC_DLT_NAME_95 As String = TZ_ATLANTIC_DLT_NAME
Private Const TZ_PACIFIC_STD_NAME_95 As String = TZ_PACIFIC_STD_NAME
Private Const TZ_PACIFIC_DLT_NAME_95 As String = TZ_PACIFIC_DLT_NAME
Private Const TZ_MOUNTAIN_STD_NAME_95 As String = TZ_MOUNTAIN_STD_NAME
Private Const TZ_MOUNTAIN_DLT_NAME_95 As String = TZ_MOUNTAIN_DLT_NAME
Private Const TZ_USMOUNTAIN_STD_NAME_95 As String = TZ_USMOUNTAIN_STD_NAME
Private Const TZ_USMOUNTAIN_DLT_NAME_95 As String = TZ_USMOUNTAIN_DLT_NAME
Private Const TZ_CENTRAL_STD_NAME_95 As String = TZ_CENTRAL_STD_NAME
Private Const TZ_CENTRAL_DLT_NAME_95 As String = TZ_CENTRAL_DLT_NAME
Private Const TZ_EASTERN_STD_NAME_95 As String = TZ_EASTERN_STD_NAME
Private Const TZ_EASTERN_DLT_NAME_95 As String = TZ_EASTERN_DLT_NAME
Private Const TZ_GMT_STD_NAME_95 As String = "GMT Standard Time"
Private Const TZ_GMT_DLT_NAME_95 As String = "GMT Daylight Time"


Public Function WeekdayOccurance(ByVal iYear As Integer, ByVal iMonth As Integer, ByVal iDayOfWeek As Integer, ByVal iOccurance As Integer) As Date
'
'Returns a Date for the specified year, month, and occurance of a weekday.
'
'iYear          Required; Integer. Number between 100 and 9999, inclusive, or a numeric expression.
'iMonth         Required; Integer. Number between 1 and 12, inclusive, or a numeric expression.
'iDayOfWeek     Required; Integer. Number between 1 and 7, inclusive, or a numeric expression.
'iOccurance     Required; Integer. Number between 1 and 5, inclusive, or a numeric expression.
'
'The iDayOfWeek argument has these settings:
'Value  Description
'  1    Sunday
'  2    Monday
'  3    Tuesday
'  4    Wednesday
'  5    Thursday
'  6    Friday
'  7    Saturday
'
'The iOccurance argument has these settings:
'Value  Description
'  1    First occurance of iDayOfWeek in iMonth
'  2    Second occurance of iDayOfWeek in iMonth
'  3    Third occurance of iDayOfWeek in iMonth
'  4    Fourth occurance of iDayOfWeek in iMonth
'  5    Fifth or last(in months where weekday only occurs four times) occurance of iDayOfWeek in iMonth
'
'Examples:
'Day                            Comment                         iYear   iMonth  iDayOfWeek  iOccurance  Result
'Thanksgiving                   Fourth Thursday in November     1998    11      5           4           11/26/1998
'Start of Daylight Savings Time First Sunday in April           1998    4       1           1           04/05/1998
'End of Daylight Savings Time   Last Sunday in October          1998    10      1           5           10/25/1998
'
Dim dtFirstDayOfMonth As Date
Dim iDaysFromMonthStart As Integer

On Error GoTo ErrRtn

        If iOccurance < 1 Then iOccurance = 1
        If iOccurance > 5 Then iOccurance = 5

        dtFirstDayOfMonth = DateSerial(iYear, iMonth, 1)
        
        'get days to first occurance of iDayOfWeek in iMonth
        iDaysFromMonthStart = (iDayOfWeek) - WeekDay(dtFirstDayOfMonth)
        If iDaysFromMonthStart < 0 Then iDaysFromMonthStart = 7 + iDaysFromMonthStart
        
        'get iOccurance occurance of iDayOfWeek in iMonth
        iDaysFromMonthStart = iDaysFromMonthStart + ((iOccurance - 1) * 7)
        WeekdayOccurance = DateAdd("d", iDaysFromMonthStart, dtFirstDayOfMonth)
        
        If iMonth <> Month(WeekdayOccurance) Then
            'this code will only execute if iOccurance equals 5 and there are only
            'four occurances of iDayOfWeek in iMonth
            
            'subtract one week to get "last" occurance of iDayOfWeek in iMonth
            WeekdayOccurance = DateAdd("ww", -1, WeekdayOccurance)
        End If
        

Xit:
    GoSub ClearObjects
Exit Function

ErrRtn:
    GoSub ClearObjects
    Err.Raise vbObjectError, MODULE & ".WeekdayOccurance", Err.Description
Resume Xit
Resume

ClearObjects:

Return

        
End Function

Public Function DateTimeZoneConvert(ByVal dtDate As Date, ByVal lFromTimeZone As TimeZoneConstants, ByRef lToTimeZone As TimeZoneConstants) As Date
'
' Returns a Date converted from one time zone to another
'
'dtDate         Required; Date.
'lFromTimeZone  Required; Long Constant. Values listed below.
'lToTimeZone    Required; Long Constant. Values listed below. Actual ToTimeZone returned.
'

'The lFromTimeZone and lToTimeZone arguments have these settings:
' Value Description
'   -2  tzcTimeZoneUnknown
'   -1  tzcTimeZoneSystem
'   0   tzcTimeZoneStandard
'   1   tzcTimeZoneDaylight
'   2   tzcTimeZoneGMT
'   4   tzcTimeZonePacific
'   6   tzcTimeZoneMountain (default)
'   8   tzcTimeZoneCentral
'   10  tzcTimeZoneEastern
'   12  tzcTimeZoneAtlantic
'
'  To get Daylight Savings Time add tzcTimeZoneDaylight to tzcTimeZoneGMT or above
'       ex: MDT = tzcTimeZoneMountain + tzcTimeZoneDaylight (value = 7)
'           MST = tzcTimeZoneMountain or tzcTimeZoneMountain + tzcTimeZoneStandard (value = 6)

'NOTE:  An error is raised if dtDate is not during Daylight Time when lFromTimeZone specifies a
'       Daylight Time.  If lToTimeZone specifies a Daylight Time and conversion of dtDate from
'       lFromTimeZone to lToTimeZone results in a date that is not during Daylight Time, the
'       date is converted to the Standard Time that corresponds with the time zone specified
'       in lToTimeZone and lToTimeZone is changed to reflect the actual time zone of the result.
'
'

Dim objTZIFrom As New CTimeZoneInfo
Dim objTZITo As New CTimeZoneInfo
Dim iFromActiveBias As Integer
Dim iToActiveBias As Integer
Dim bDaylight As Boolean

On Error GoTo ErrRtn

    With objTZIFrom
        .TimeZoneID = lFromTimeZone
        bDaylight = (lFromTimeZone Mod 2 = tzcTimeZoneDaylight) 'specified a Daylight Time
        If .DateInDaylightRange(dtDate) = True Then
            'Actually in Daylight Time range
            
            If (bDaylight) Then
                'Daylight TimeZone
                iFromActiveBias = .Bias + .DaylightBias
            Else
                'Standard TimeZone is always valid
                iFromActiveBias = .Bias + .StandardBias
            End If
            
        Else
            'Not in Daylight Time range
            
            If (bDaylight) Then
                'Specified Daylight Time, but outside range
                
                Err.Raise vbObjectError, , "Date was specified as being in daylight savings time, but Date falls outside of daylight savings range."
            Else
                'Specified Standard Time which is always valid
                iFromActiveBias = .Bias + .StandardBias
            End If
        End If
    End With
    
    
    'this check is done here so that it occurs after the check that dtDate is during daylight time range
    'if specified as datelight time so the error will be raised.
    If lFromTimeZone = lToTimeZone Then
        DateTimeZoneConvert = dtDate
        GoTo Xit
    End If
    
    With objTZITo
        .TimeZoneID = lToTimeZone
        bDaylight = (lToTimeZone Mod 2 = tzcTimeZoneDaylight) 'specified a Daylight Time
        If (bDaylight) Then
            'Daylight TimeZone
            iToActiveBias = .Bias + .DaylightBias
        Else
            'Standard TimeZone
            iToActiveBias = .Bias + .StandardBias
        End If
        
        'convert from "from" timezone to "to" timezone
        DateTimeZoneConvert = DateAdd("n", iFromActiveBias - iToActiveBias, dtDate)
        
        'only worry about this if "to" timezone is daylight
        If (bDaylight) Then
            'Specified as Daylight Time
            If .DateInDaylightRange(DateTimeZoneConvert) = False Then
                'resulting date is not during daylight range, but a "to" timezone was specified as daylight
                'convert back to standard
                iToActiveBias = .Bias + .StandardBias
                DateTimeZoneConvert = DateAdd("n", iFromActiveBias - iToActiveBias, dtDate)
                
                'convert to standard time to reflect what timezone the result is in
                'lToTimeZone is a ByRef arg to pick up this change
                lToTimeZone = lToTimeZone - tzcTimeZoneDaylight
            End If
        End If
    End With
        

Xit:
    GoSub ClearObjects
Exit Function

ErrRtn:
    GoSub ClearObjects
    Err.Raise vbObjectError, MODULE & ".DateTimeZoneConvert", Err.Description
Resume Xit
Resume

ClearObjects:

Return

End Function

Public Function GetTimeZoneDisplayNames() As Variant
Dim oSystem As New CSystem
Dim objRegKey As RegKey
Dim sKey As String
Dim objItem As RegKey
Dim sDisplayName As String
Dim lID As Long
Dim vDisplayNamesAndIDs() As Variant
Dim x As Long

On Error GoTo ErrRtn

    If oSystem.Platform = emsOSPlatformWindowsNT Then
        sKey = TZ_REG_KEY_NT
    Else
        sKey = TZ_REG_KEY_95
    End If

    Set objRegKey = RegKeyFromString("\HKEY_LOCAL_MACHINE\" & sKey)
    
    x = 0

    For Each objItem In objRegKey.SubKeys
        ReDim Preserve vDisplayNamesAndIDs(0 To 1, x)

        sDisplayName = objItem.Values("Display").Value
        lID = GetIDFromShortName(objItem.Values("Std").Value)

        vDisplayNamesAndIDs(0, x) = sDisplayName
        vDisplayNamesAndIDs(1, x) = lID
        x = x + 1
   Next

    GetTimeZoneDisplayNames = vDisplayNamesAndIDs

Xit:
    GoSub ClearObjects
Exit Function

ErrRtn:
    GoSub ClearObjects
    Err.Raise vbObjectError, MODULE & ".GetTimeZoneDisplayNames", Err.Description
Resume Xit
Resume

ClearObjects:

Return


End Function


Public Function GetIDFromShortName(ByVal sStandardOrDaylightName As String) As TimeZoneConstants
On Error GoTo ErrRtn

    Select Case sStandardOrDaylightName
        Case TZ_ATLANTIC_STD_NAME, TZ_ATLANTIC_STD_NAME_95
            GetIDFromShortName = tzcTimeZoneAtlantic
        Case TZ_CENTRAL_STD_NAME, TZ_CENTRAL_STD_NAME_95
            GetIDFromShortName = tzcTimeZoneCentral
        Case TZ_EASTERN_STD_NAME, TZ_EASTERN_STD_NAME_95
            GetIDFromShortName = tzcTimeZoneEastern
        Case TZ_MOUNTAIN_STD_NAME, TZ_MOUNTAIN_STD_NAME_95
            GetIDFromShortName = tzcTimeZoneMountain
        Case TZ_USMOUNTAIN_STD_NAME, TZ_USMOUNTAIN_STD_NAME_95
            GetIDFromShortName = tzcTimeZoneUSMountain
        Case TZ_PACIFIC_STD_NAME, TZ_PACIFIC_STD_NAME_95
            GetIDFromShortName = tzcTimeZonePacific
        Case TZ_GMT_STD_NAME, TZ_GMT_STD_NAME_95
            GetIDFromShortName = tzcTimeZoneGMT
            
        Case TZ_ATLANTIC_DLT_NAME, TZ_ATLANTIC_DLT_NAME_95
            GetIDFromShortName = tzcTimeZoneAtlantic + tzcTimeZoneDaylight
        Case TZ_CENTRAL_DLT_NAME, TZ_CENTRAL_DLT_NAME_95
            GetIDFromShortName = tzcTimeZoneCentral + tzcTimeZoneDaylight
        Case TZ_EASTERN_DLT_NAME, TZ_EASTERN_DLT_NAME_95
            GetIDFromShortName = tzcTimeZoneEastern + tzcTimeZoneDaylight
        Case TZ_MOUNTAIN_DLT_NAME, TZ_MOUNTAIN_DLT_NAME_95
            GetIDFromShortName = tzcTimeZoneMountain + tzcTimeZoneDaylight
        Case TZ_USMOUNTAIN_DLT_NAME, TZ_USMOUNTAIN_DLT_NAME_95
            GetIDFromShortName = tzcTimeZoneUSMountain + tzcTimeZoneDaylight
        Case TZ_PACIFIC_DLT_NAME, TZ_PACIFIC_DLT_NAME_95
            GetIDFromShortName = tzcTimeZonePacific + tzcTimeZoneDaylight
        Case TZ_GMT_DLT_NAME, TZ_GMT_DLT_NAME_95
            GetIDFromShortName = tzcTimeZoneGMT + tzcTimeZoneDaylight
            
        Case Else
            GetIDFromShortName = tzcTimeZoneUnknown
    End Select
    

Xit:
    GoSub ClearObjects
Exit Function

ErrRtn:
    GoSub ClearObjects
    Err.Raise vbObjectError, MODULE & ".GetIDFromShortName", Err.Description
Resume Xit
Resume

ClearObjects:

Return
    
End Function

Public Function GetTimeZoneRegKeyFromID(ByVal aTimeZone As TimeZoneConstants) As String
' Given an enum value from TimeZoneConstants, return the registry key to access time zone information

Dim oSystem As New CSystem
On Error GoTo ErrRtn

    ' key names vary from platform to platform
    If oSystem.Platform = emsOSPlatformWindowsNT Then
    
        Select Case aTimeZone
            Case tzcTimeZoneAtlantic, tzcTimeZoneAtlantic + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_ATLANTIC
                
            Case tzcTimeZonePacific, tzcTimeZonePacific + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_PACIFIC
                
            Case tzcTimeZoneMountain, tzcTimeZoneMountain + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_MOUNTAIN
        
            Case tzcTimeZoneUSMountain, tzcTimeZoneUSMountain + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_USMOUNTAIN
                   
            Case tzcTimeZoneCentral, tzcTimeZoneCentral + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_CENTRAL
        
            Case tzcTimeZoneEastern, tzcTimeZoneEastern + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_EASTERN
            
            Case tzcTimeZoneGMT, tzcTimeZoneGMT + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_GMT
        End Select
        
    Else

        Select Case aTimeZone
            Case tzcTimeZoneAtlantic, tzcTimeZoneAtlantic + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_ATLANTIC_95
                
            Case tzcTimeZonePacific, tzcTimeZonePacific + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_PACIFIC_95
                
            Case tzcTimeZoneMountain, tzcTimeZoneMountain + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_MOUNTAIN_95
        
            Case tzcTimeZoneUSMountain, tzcTimeZoneUSMountain + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_USMOUNTAIN_95
           
           Case tzcTimeZoneCentral, tzcTimeZoneCentral + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_CENTRAL_95
        
            Case tzcTimeZoneEastern, tzcTimeZoneEastern + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_EASTERN_95
            
            Case tzcTimeZoneGMT, tzcTimeZoneGMT + tzcTimeZoneDaylight
                GetTimeZoneRegKeyFromID = TZ_REG_GMT
        
        
        End Select
        
    End If

Xit:
    GoSub ClearObjects
    Exit Function

ErrRtn:
    GoSub ClearObjects
    Err.Raise vbObjectError, MODULE & ".GetTimeZoneRegKeyFromID", Err.Description
    Resume Xit
    Resume

ClearObjects:
    Return

    
End Function

